
from ovirtcli.command.action import ActionCommand
from ovirtcli.command.add import AddCommand
from ovirtcli.command.connect import ConnectCommand
from ovirtcli.command.console import ConsoleCommand
from ovirtcli.command.remove import RemoveCommand
from ovirtcli.command.disconnect import DisconnectCommand
from ovirtcli.command.help import HelpCommand
from ovirtcli.command.list import ListCommand
from ovirtcli.command.ping import PingCommand
from ovirtcli.command.show import ShowCommand
from ovirtcli.command.status import StatusCommand
from ovirtcli.command.update import UpdateCommand
from ovirtcli.command.history import HistoryCommand
